        subroutine adjust

!       SUBROUTINE THAT ADJUSTS INPUT VELOCITY TO MEET OUTPUT BOUNDARY 
!       CONDITIONS

!       DECLARE VARIABLES USED IN THIS SUBROUTINE

        use precis_param
        use Common1
        use Common2
        use Common3
        use ints
        use melt
        use adjust_vars
        implicit none

        integer j
        character answer*1

!       IF THIS IS THE FIRST RUN, INITIALIZE RMINV, RMAXO
        if (iruns.eq.1) then
            rminv = 5.0e+05_ip           !minimum possible velocity
            rmax0 = 0.e00_ip             !maximum possible velocity
        end if

!        write (6,2043) iruns,rminv,rmax0
!        write (9,2043) iruns,rminv,rmax0
!2043    format(/,2x,'Starting adjust. iruns = ',i3,' rminv=',e8.3, &
!                    '   rmax0=',e8.3)

        vellast = vel(1)

!       write (10,4512) vel(1), z(i), mach(i), pres(i)/1.0d+06, iruns,
!     *         rminv, rmax0
!4512   format (4f12.4,i6,2f12.4)

!*********************************************************************

!       SOLUTIONS FOR Z=ZFINAL

!*********************************************************************

        if (z(i).eq.zfinal) then

!       IF Z(I)=ZFINAL AND P>PFINAL, SEE IF THERE ARE PREVIOUS CALCULATIONS
!       RESULTING IN M>1 OR P<PFINAL.  IF SO, INCREASE VEL(1) BY 0.2 
!       TIMES THE DIFFERENCE BETWEEN THE MINIMUM V1M OR V1P AND THE 
!       MAXIMUM V1O

!       VARIABLES:
!         V1O = INPUT VELOCITY FOR A RUN THAT RESULTED IN P>PFINAL, Z=ZFINAL
!         V1P =  "       "      "  "  "    "     "     "  P<PFINAL, Z<ZFINAL
!         V1M =  "       "      "  "  "    "     "     "  M>1, Z<ZFINAL
!         IO =  NUMBER OF RUNS THAT RESULTED IN P>PFINAL, Z=ZFINAL
!         IPP =     "   "   "     "     "      " P<PFINAL, Z<ZFINAL
!         IM =     "   "   "     "     "      " M>1, Z<ZFINAL
!       

          if (pres(i).gt.pfinal) then
            write (6,*) 'exit pressure > pfinal and M < 1'
            write (9,*) 'exit pressure > pfinal and M < 1'
            v1o(io) = vel(1)
             if (v1o(io).gt.rmax0) rmax0 = v1o(io)
            io = io+1

!      FIND MINIMUM OF V1P AND V1M

!           rminv = 5.d+05
!           do 150 j=1,im-1
!150        rminv=min(rminv,v1m(j))
!           do 160 j=1,ipp-1
!160        rminv=min(rminv,v1p(j))

!       FIND MAXIMUM OF V1O
!
!           rmax0 = 0.
!           do 170 j=1,io-1
!170        rmax0=max(rmax0,v1o(j))

!       CALCULATE NEW VALUE OF VEL(1)
             if (rminv.lt.1.e+05_ip) then
              vel(1) = rmax0 + 0.5*(rminv-rmax0)
             else
              vel(1) = 1.5*vel(1)
            end if
            write(6,1043) iruns, rminv, rmax0
            write(9,1043) iruns, rminv, rmax0
1043        format(5x,'iruns=',i4,'   rminv=',e8.2,'   rmax0=',e8.2)
            go to 500

!       IF Z=ZFINAL AND THE FINAL PRESSURE IS <(ZFINAL-.0001) MPA, 
!       ADJUST THE INITIAL VELOCITY SO THAT IT'S HALFWAY BETWEEN 
!       THE CURRENT VEL(1) AND THE MAXIMUM V1O

          else
            v1p(ipp) = vel(1)
!           do 171 j=1,io-1
!171        rmax0=max(rmax0,v1o(j))
            vel(1) = rmax0 + 0.5*(vel(1)-rmax0)
            if (vel(1).gt.rmax0) rmax0 = vel(1)
            if (vel(1).lt.0.001) vel(1) = 0.001
            ipp = ipp+1
            go to 500
          end if
        end if

!*********************************************************************

!       SOLUTIONS FOR Z < ZFINAL

!*********************************************************************

!       IF M>1, THE RELATIONSHIP BETWEEN (Z(I)-ZFINAL) AND VEL(1) SHOULD BE
!       ALMOST LINEAR.  THEREFORE, IF WE HAVE RUN AT LEAST
!       TWO ITERATIONS WHERE VEL(1) AND Z(I) HAVE BEEN OBTAINED,
!       WE CAN USE THOSE VALUES TO DEFINE THE LINE RELATING Z(I) AND
!       VEL(1), AND CALCULATE THE VEL(1) NEEDED TO OBTAIN M=1 AT
!       Z(I) = ZFINAL.   THIS VALUE IS VEL(1)=-INT/SLOPE, WHERE INT IS THE
!       Z(I) INTERCEPT OF THE Z(I) VS VEL(1) LINE, AND SLOPE IS THE
!       SLOPE OF THE LINE.

!       IT TURNS OUT THAT, IF ONE ASSUMES THE RELATIONSHIP
!       BETWEEN (Z(I)-ZFINAL) AND VEL(1) TO BE PERFECTLY LINEAR, ONE OVERSHOOTS
!       AND ENDS UP BACK IN THE M<1 FIELD.  THEREFORE, I ADD 10% OF 
!       THE DIFFERENCE BETWEEN THE OLD VEL(1) AND THAT OBTAINED FROM
!       VEL(1)=-INT/SLOPE, TO MAKE SURE THAT VEL(1) STAYS WITHIN
!       THE M>=1 FIELD.

        if (mach(i).gt.1.000) then
          write (6,1425)
          write (9,1425)
1425      format(5x,' mach number > 1.  adjusting initial velocity')
          zlm(im) = z(i)-zfinal
          v1m(im) = vel(1)
          if (v1m(im).lt.rminv) rminv = v1m(im)

!       CALCULATE SLOPE AND INTERCEPT OF V(1) VS (Z(I)-ZFINAL) LINE,
!       AND VALUE OF V(1) ALONG THIS LINE WHEN Z(I)=0.  

          if (im.ge.2) then
            slope = (zlm(im)-zlm(im-1))/(v1m(im)-v1m(im-1))
            int1 = zlm(im)-slope*v1m(im)
            vel(1) = -int1/slope + 0.1*(min(v1m(im),v1m(im-1))+ &
                        int1/slope)
            if ((vel(1).gt.rminv).or.(vel(1).lt.rmax0)) then
                         vel(1)=rmax0+0.5*(rminv-rmax0)
            end if
           else 
            vel(1) = -vel(1)*(z(i)-z(1))/z(1) 
          end if
          im = im+1

!       WRITE OUT RESULTS
!       write (6,1042) im, slope, int1, vel(1),zlm(im-1),
!    *                 zlm(im-2),v1m(im-1),v1m(im-2)
!1042    format(2x,'im=',i3,'  slope=',e8.3,'  int1=',e8.3,/,
!     *            3x,' vel(1)=',f8.3,/,
!     *            3x,'zlm(im)=',f8.3,'  zlm(im-1)',f8.3,/,
!     *            3x,'vlm(im)=',f8.3,'  vlm(im-1)',f8.3,/,
!     *            3x,'shall we continue?'$)
!        read (5,2044) answer
!2044    format(a1)
!        if (answer.ne.'y') then
!           write (6,*) 'program stopped'
!           stop
!        end if

!       FOR P<PFINAL, ADJUST V(1).  ASSUME, AS WHEN M>1, THAT THE
!       RELATIONSHIP BETWEEN V(1) AND Z(I) IS LINEAR.  CALCULATE V(1)
!       ALONG THIS LINE FOR Z(I)=ZFINAL.

        else if (pres(i).lt.pfinal) then
          write (6,5635) 
          write (9,5635)
5635            format(5x,'pressure is less than pfinal.', &
                        '  adjusting initial velocity')
            zlp(ipp) = z(i)-zfinal
            v1p(ipp) = vel(1)
            if (vel(1).lt.rminv) rminv = vel(1)
            if (ipp.ge.2) then
              slope = (zlp(ip)-zlp(ipp-1))/(v1p(ipp)-v1p(ipp-1))
              int1 = zlp(ipp)-slope*v1p(ipp)

!                 THE FOLLOWING STATEMENTS AVOID FLOATING-POINT DIVIDE ERRORS
                  crit = -9999.
                  if (slope.ne.0.) crit=-int1/slope
              if ((crit.le.0.).or.(crit.gt.vel(1))) then
                    vel(1) = vel(1)/2.
               else
                vel(1) = -int1/slope
              end if
            else
            vel(1) = vel(1)*(z(i)-z(1))/(zfinal-z(1)) 
          end if
          if (vel(1).le.0.001) vel(1)=0.001
          if (vel(1).lt.rmax0) vel(1) = (rminv+rmax0)/2.
          ipp = ipp+1
        end if

!       IF THE MINIMUM OF V1P OR V1M AND THE MAXIMUM OF V1O DONT DIFFER BY 
!       MORE THAN .001 m/s, (AND THE SOLUTION LIES NEAR THE SURFACE) THEN 
!       CALL THE SOLUTION GOOD.

500     continue

        if ((((rmax0.gt.0.).and.(abs(rminv-rmax0).lt.0.001)).or. &
        (abs(vel(1)-vellast).lt.0.001)).and.(z(i).gt.(zfinal-5.))) &
        then
              write(6,5465)
              write(9,5465)
5465          format(/,5x,'initial velocities for succeeding runs', &
                          ' differ by <0.001 m/s. Program stopped.',/)
              write(6,5466) rmax0,rminv,vel(1),vellast
              write(9,5466) rmax0,rminv,vel(1),vellast
5466          format('rmax0 = ',e9.3,4x,'rminv = ',e9.3,4x, &
                     'vel(1) = ',f6.3,4x,'vellast = ',f6.3)
              iend=1
              return
        end if

!       RE-INITIALIZE VARIABLES AND WRITE OUT NEW INITIAL VELOCITY

        do 100 j=1,i
          re(j) = 0.
          rho(j) = 0.
          mach(j) = 0.
          vesic(j) = 0.
          visc(j) = 0.
100     continue
        do 200 j=2,i
          vel(j) = 0.
          pres(j) = 0.
          z(j) = 0.
200     continue
        i = 1
        write (6,1044) vel(1)
        write (9,1044) vel(1)
1044    format (/,' trying new input velocity ',f10.5,' m/s',/)

!        write (6,1045) rminv, rmax0
!        write (9,1045) rminv, rmax0
!1045    format(3x,'finishing adjust.  rminv=',e8.3,'  rmax0=',e8.3)

!        write(6,*) 'Continue?'
!        read(5,'(a1)') answer
!        if (answer.eq.'n') stop
        return
        end
