        subroutine derivs(znow,yp,grad)

!       SUBROUTINE TO CALCULATE PRESSURE GRADIENT BETWEEN THE PREVIOUS
!       AND THE CURRENT DEPTH.

!       DECLARE VARIABLES USED IN THIS SUBROUTINE

        use precis_param
        use Common1
        use Common2
        use ints
        use melt
        use water
        implicit none
        real(kind=ip) ::  blkmix,bubrad,bubvol,&
               dvdrmax,dvdth2o,edot,grad, &
               p,term1,yp,znow
        real(kind=ip) ::  xcomp(10)
        real(kind=ip) :: dgdxw,dvmdt,gamma1,gwm,hwm,pbars,swm

           if (icalc.eq.1) then
             p = yp
             diam = dfirst + (dfinal-dfirst)*(znow-z(1))/(zfinal-z(1))
             xsarea = 3.14159 * diam**2/4.
            else
             xsarea = yp
             if (xsarea.lt.0.)  xsarea = 0.0001
             diam = 2.*sqrt(xsarea/3.14159)
             p = pfinal + dpdz*(znow-zfinal)
           end if

!       DETERMINE NEW MASS FRACTION GAS

        if ((ifrag.eq.1).and.(ives.eq.2)) go to 1265
        if (p.ge.0.) then
                dh2o = .01*s*(p/1.0d+06)**beta
            if (dh2o.le.0.) dh2o = 1.0d-03
                mm = msil/(1.-dh2o)
                mg = 1.-mxtls-mm
            if (mg.lt.0.) then
                        mg=0.
                        mm = 1.-mxtls
                        dh2o = mw/mm
            end if
          else
            dh2o = 1.0d-03
                mg = mw*(1.-dh2o)/msil
                mm = 1.-mg-mxtls
        end if
1265    continue

!       DETERMINE PROPERTIES OF MAGMA AND GAS

        if (p.gt.0.) then
          pbars = p/1.0e05
          call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
                  entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
                  dvmdt)
          call haarsi(temp,pbars,cpH2O,cvH2O,gH2O,hH2O,sH2O, &
                vH2O,blkgas,dvdth2o)
        end if

        cp      = cpH2O/mh2o
        cv      = cvH2O/mh2o
        rhof = (1.0D+06*.0180152)/vH2O
        gamma1 = cp/cv

!       CALCULATE MIXTURE DENSITY, VFGAS
        rhomix = 1./(mm/rhom + mg/rhof + mxtls/rhoxtls)
        vfgas = mg*rhomix/rhof 
        if (vfgas.gt.1.0) vfgas = 0.99999d+00
        vm = mm*rhomix/rhom
        vxtls = mxtls*rhomix/rhoxtls

!       CALCULATE NEW VELOCITY, VELOCITY GRADIENTS, STRAIN RATES
        v = mdot/(rhomix*xsarea)
        if ((i.gt.1).and.(znow.gt.z(i))) then
!               extensional strain rate in z direction
                dvdz = (v-vel(i))/(znow-z(i))
          else if (i.gt.1) then
                dvdz = (v-vel(i-1))/(znow-z(i-1))
          else
                dvdz = 0.
        end if

        if (ifrag.eq.0) then              !shear-strain rate at conduit wall
                dvdrmax = -2.*v/diam
          else
                dvdrmax = 0.
        end if
        emax = dvdz/2. + sqrt((dvdz/2.)**2 + dvdrmax**2)    !max strain rate

!       CALCULATE BULK MODULUS AND SONIC VELOCITY
        blkmix = 1./(vfgas/blkgas + (1.-vfgas)*(1.-0.01*xtls)/blkmag +  &
                (1.-vfgas)*0.01*xtls/1.0d+11)
        if (blkmix.lt.0.) then
               write (6,*) 'blkmix < 0'
               stop
        end if
      sv = sqrt(blkmix/rhomix)
          
!       CALCULATE VISCOSITY
        if (ifrag.eq.0) then
                call vis
!               CALCULATE CAPILLARY NUMBER, VEX
                if (vfgas.gt.0.) then                           
                        edot = 4.*v/(3.*diam)
                        bubvol = vfgas/(bubnum*vm)
                        bubrad = 10.**((log10(3.*bubvol/(4.*3.14159))) &
                                       /3.)
                        ca = etamag * edot * bubrad / 3.4d-01
                        vex = (2./3.14159) * atan(5.*log10(ca))
                  else
                        Ca = 1.
                        vex = 0.
                end if
                eta = etamag*(1.-vfgas)**vex
         else
                if (vfgas.gt.0.40) then
                  etagas = (5.3d-05 * (1.-(1.-vfgas)/0.62)**(-1.5))
                 else
                  etagas = 5.3d-05
            end if
                eta = etagas
            Ca = 1.
        end if

!       CALCULATE REYNOLDS, NUMBER, FRICTION FACTOR
        rey = rhomix*v*diam/eta
        f = 16./rey + fo

!       CALCULATE MACH NUMBER

        m = v/sv

!       CALCULATE GRADIENT IN PRESSURE OR XSAREA

        if (icalc.eq.1) then
           term1 = 2.*f*v**2/diam
           grad = (rhomix*(-g - 2.*f*v**2/diam + v**2/xsarea * dadz))/ &
                (1.-m**2)
           dpdz = grad
          else
                   grad = xsarea/v**2 * ( dpdz*(1.-m**2)/rhomix + g + &
                (2.*f*v**2)/diam )
           dadz = grad
        end if

        return
        end
