      program confort

!      PROGRAM THAT CALCULATES FLOW PROPERTIES IN ERUPTIVE CONDUITS 
!      THIS PROGRAM WAS WRITTEN BY LARRY G. MASTIN, U.S. GEOLOGICAL
!      SURVEY, CASCADES VOLCANO OBSERVATORY, 5400 MACARTHUR BLVD.,
!      VANCOUVER WA 98661 (360) 993-8925, lgmastin@usgs.gov.  

!      INFORMATION ABOUT IT CAN BE OBTAINED THROUGH THE 
!      CASCADES VOLCANO OBSERVATORY WEB SITE:

!      HTTP://VULCAN.WR.USGS.GOV

!      DECLARE VARIABLES

      use precis_param
      use Common1
      use Common2
      use Common3
      use outdata
      use ints
      use water
      use melt
      
      implicit none
      real(kind=ip)  xcomp(10)
      real(kind=ip)  cptotal, &
             cxtls,deltah,deltat,&
             dgdxw,dh,dvdth2o,dvmdt,dznext, &
             dzout,edot, fragval,gmix,gwm, &
             h2o,hnew,ho,hwm,mmlast, &
             mxhat,p,pbars,perr,pi,pmax,pmin,pout, &
             smod,stresscon, &
             swm,t1,tbeam,temp2,temp2c, &
             vavg,vmax,wtdepth,wtmin, &
             xserr,zfrag,zin
      integer ifragtype,iin,imax,in,iparam, &
             itemp,itnum,itotal,ivar(12), &
             ivars,ivt,iwrite,ix
      character title(27)*10, outfile*40, icalctxt*4, answer*1
      external derivs

      data title/'      area','    mach #','   p (MPa)', &
             '    log Re','   density','  time (s)',' vel (m/s)', &
             '     vfgas','  log visc','     z (m)','     da/dz', &
             '    log(p)','dpdt,MPa/s','     dz(m)', &
             '         f','     gamma',' mgas/mtot',' mmag/mtot', &
             ' R (J/m K)','  gas dens',' sonic vel','   temp(C)', &
             ' h (kJ/kg)','        cp','    radius',' dh2o(wt%)', &
             ' cm(J/kgK)'/

!      NOTE CONFORT VERSION NUMBER
      write (6,1009)
1009      format('Confort version 1.0.4.',/, &
             'last revised May, 2003 by Larry G. Mastin',/, &
             'U.S. Geological Survey lgmastin@usgs.gov',//)

!      SET VALUES FOR CONSTANTS: BELOW ARE MOLECULAR WEIGHTS (KG/KMOLE)

      fo = 0.0025
      
!      OPEN INPUT AND OUTPUT FILES

      open (unit=8,file='conin.txt')
      
!      READ INPUT DATA

      read (8,*)
      read (8,*) outfile

      open (unit=9,file=outfile)

      read (8,*) icalctxt
      read (8,*) pmax, pmin
      read (8,*) ivt
      read (8,*) vel(1)
      read (8,*) t1, itemp
      read (8,*) cxtls
      read (8,*) h2o
      read (8,*) ives
      read (8,*) z(1), zfinal
      read (8,*) dfirst, dfinal
      read (8,*) g

      read (8,*) SiO2
      read (8,*) Al2O3
      read (8,*) Fe2O3
      read (8,*) FeO
      read (8,*) MgO
      read (8,*) CaO
      read (8,*) TiO2
      read (8,*) Na2O
      read (8,*) K2O
      read (8,*) xtls, rhoxtls
      read (8,*) fragval      !vesicularity at fragmentation
!      READ FRAGMENTATION CRITERION:
!      ifragtype = 1: fragmentation at vfgas>.01*fragval
!      2: fragmentation at eta * dvdz > 25 MPa
!      3: fragmentation at eta * dvdz > 0.01*smod for vesicular solid
!      4: fragmentation at eta * dvdz * stress concentration around bubble wall > 250 MPa
      read (8,*) ifragtype

!      SET INPUT PARAMETERS
      pres(1) = pmax*1.0e+06
      pfinal = pmin*1.0e+06
      if (z(1).gt.0.) z(1) = -z(1)
      if (zfinal.gt.0.) zfinal = -zfinal
      wtoxan(1) = SiO2
      wtoxan(2) = Al2O3
      wtoxan(3) = Fe2O3
      wtoxan(4) = FeO
      wtoxan(5) = MgO
      wtoxan(6) = CaO
      wtoxan(7) = TiO2
      wtoxan(8) = Na2O
      wtoxan(9) = K2O
      diam = dfirst
      i = 1
      iend = 0
      im = 1
      ipp = 1
      io = 1
      iruns = 0
      mw = 1.d-02 * h2o
      mh2o = 0.0180152
      pi = 3.14159d+00
      r = 8.314
      area(1) = pi * (diam/2.)**2
      temp = t1 + 273.15
      xsarea = area(1)

      if (icalctxt.eq.'pgrd') icalc=2
      if (icalctxt.eq.'diam') icalc=1

      bubnum = 10.**(0.2*SiO2)      !estimate bubble number density

!      DETERMINE VALUES OF S AND BETA
      call solub(pmin,pmax,temp,wtoxan,s,beta)

!      CALL CHECKER TO MAKE SURE INPUT VALUES ARE REASONABLE
      call checker(icalctxt,pres(1),pfinal,ivt,vel(1),t1,itemp, &
             cxtls,h2o,s,beta,ives,vex,z(1),zfinal,dfirst, &
             dfinal,g,wtoxan,xtls,rhoxtls)

!      READ PARAMETERS TO WRITE TO OUTPUT FILE
      do 1039 iparam=1,16
1039      read (8,*)

      ivars=0
      ivar=0
      do iin=1,27
         read (8,31,err=1041) ix
31       format (i1)
         if (ix.le.0) go to 1041
         if (ivar(ix).ne.0) then
            write(6,*) 'output ix=',ix,', title(ivar(ix))=',title(ivar(ix))
            write (6,4823) ix, title(ivar(ix)), title(iin)
4823        format (4x,'Output to column ',i1, &
               ' has been specified for TWO variables:',/, &
               10x,a10,/,10x,a10,/, &
               5x,'Please edit the input file to correct the problem.',/, &
               5x,'Remember to enter the output column number in the ',/, &
               5x,'FIRST column of the appropriate line in the input ', &
               'file.')
            stop
         end if
         ivar(ix)=iin
         ivars = ivars+1
1041  end do

!      MAKE SURE OUTPUT VARIABLES ARE ENTERED CORRECTLY
      write(6,*) 'making sure variables are entered correctly'
      if (ivars.ne.7) then
      write (6,4298)
4298      format(/,2x,'You have entered your output variables ', &
             'incorrectly.  They are:')
      do ix=1,7
         if (ivar(ix).gt.0) then
            write (6,4290) ix, title(ivar(ix))
4290        format(5x,'column ',i2,a10)
          else
            write (6,4290) ix, ' not given'
          end if
      end do
      write (6,4390)
4390  format(2x,'Please edit the input file, entering seven ' &
             'output variables, and start again.',/, &
             2x,'Remember to enter the output column number in the ' &
             'FIRST column of each',/,2x,'appropriate line.')
      stop
      end if

!      SET INITIAL VALUES

!      THE VARIABLE EPS IS A CRITERION USED BY THE SUBROUTINE RKQC TO
!      DETERMINE WHETHER TO INCREASE OR DECREASE STEP SIZES. UP TO A
!      POINT, INCREASING THE VALUE OF EPS WILL INCREASE STEP SIZES,
!      MAKING THE MODEL RUN MORE EFFICIENTLY WITHOUT SACRIFICING ACCURACY.
!      INCREASING EPS BEYOND THAT POINT MAKES THE MODEL UNSTABLE, OR
!      DOES NOT SIGNIFICANTLY INCREASE STEP SIZES.  THE VALUES OF EPS
!      USED HERE WERE FOUND EMPIRICALLY TO BE THE MAXIMUM VALUES THAT
!      CAN BE USED WITHOUT MAKING THE MODEL RUN UNSTABLY.
!      
      if (icalc.eq.2) then
      dpdz = (pfinal-pres(1))/(zfinal-z(1))
      ivt = 1
      eps = 2.0d-02
      imax = 10000
      else
      dadz = (dfinal-dfirst)/(zfinal-z(1))
      eps = 3.0d-07
      imax = 10000
      end if

!      CALCULATE AMOUNT OF DISSOLVED WATER IN MELT AT SATURATION
      dh2o = 0.01*s*(pres(1)/1.0d+06)**beta

!      CALL MULTISI, HAARSI TO CALCULATE THERMO PROPS.
      pbars = pres(1)/1.0e05
      call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
             entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
             dvmdt)
      call haarsi(temp,pbars,cpH2O,cvH2O,gH2O,hH2O, &
             sH2O,vH2O,blkgas,dvdth2o)

!      CALCULATE MASS FRACTIONS
3025      mxhat = 0.01*xtls*rhoxtls /  &
             (0.01*xtls*rhoxtls + (1.-0.01*xtls)*rhom)
      mg = (mw - (1.-mxhat)*dh2o) / (1. - (1.-mxhat)*dh2o)
      mxtls = (1.-mg)*mxhat
      msil = 1.d00 - mw - mxtls
      mm = 1. - mg - mxtls

!      CHECK TO SEE IF MELT IS SATURATED.  IF NOT, RECALCULATE PROPERTIES.
      if (mw.lt.(mm*dh2o)) then
      mm = 1.0d00-mxtls
      dh2o = mw/mm
      mg = 0.
      do 1045 while (abs(mm-mmlast).gt.1.0d-06)
      mmlast = mm
      pbars = p/1.0e05
      call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
             entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
             dvmdt)

      mxhat = 0.01*xtls*rhoxtls / &
             (0.01*xtls*rhoxtls + (1.-0.01*xtls)*rhom)
      mxtls = mxhat
      msil = 1.d00 - mw - mxtls
      mm = 1.d00 - mxtls
1045      continue
      end if

      rhomix = 1./(mg/rhof + mm/rhom + mxtls/rhoxtls)      !density of mixture
      vfgas = mg*rhomix/rhof      !volume fraction gas
      vm      = mm*rhomix/rhom      !volume fraction melt
      vxtls = mxtls*rhomix/rhoxtls      !volume fraction crystals

!      CALCULATE GAS PROPERTIES
      cp      = cpH2O/mh2o
      cv      = cvH2O/mh2o
      rhof = (1.0D+06*mh2o)/vH2O
      gamma = cpH2O/cvH2O
      hf = hH2o/mh2o

!      WRITE OUT INITIAL CONDITIONS
      write (9,153)
153      format (10x,'using program "conflow"')
      write (6,134) vel(1), rhom, t1, h2o,  &
             -z(1), -zfinal
      write (9,134) vel(1), rhom, t1, h2o,  &
             -z(1), -zfinal
134      format (/,10x,'INPUT VALUES:',//, &
             10x,' input velocity = ',f8.4,' m/s',/, &
             10x,' magma density=',f7.0,' kg/m3 (calculated)',/, &
             10x,' input temperature =',f6.0,' degrees Celsius',/, &
             10x,' initial dissolved h2o=',f6.3,' wt %',/, &
             10x,' depth at base of conduit=',f10.4,' m',/, &
             10x,' depth at top of conduit=',f10.4,' m')

      write (6,9342) SiO2, Al2O3, Fe2O3, FeO, MgO, CaO, &
             TiO2, Na2O, K2O, xtls, rhoxtls
      write (9,9342) SiO2, Al2O3, Fe2O3, FeO, MgO, CaO, &
             TiO2, Na2O, K2O, xtls, rhoxtls
9342      format (/,10x,'MAGMA COMPOSITION:',//, &
             10X,F7.3,'      SIO2 (wt% anhydrous)',/, &
             10x,f7.3,'      Al2O3',/,10x,f7.3,'      Fe2O3',/, &
             10x,f7.3,'      FeO',/,10x,f7.3,'      MgO',/, &
             10x,f7.3,'      CaO',/,10x,f7.3,'      TiO2',/, &
             10x,f7.3,'      Na2O',/,10x,f7.3,'      K2O',/, &
             10x,f7.3,'      crystal content (vol% of melt)',/, &
             10x,f9.3,'      crystal density (kg/m3)',/)

      if (itemp.eq.2) then
      write (6,1638)
      write (9,1638)
1638      format (/,10x,'calculating temperature change')
      else
      write (6,1639)
      write (9,1639)
1639      format (/,10x,'assuming constant temperature')
      end if

      if (icalc.eq.1) then
      write (6,1538) diam,dfinal, pres(1)/1.0d+06
      write (9,1538) diam,dfinal,  pres(1)/1.0d+06
1538      format (/,10x,'specified conduit diameter:',/, &
             10x,'diameter at base = ',f10.2,' meters',/, &
             10x,'diameter at top = ',f10.2,' meters',/, &
             10x,'input pressure = ',f6.2,' MPa')
      else
      write (6,1539) -dpdz/1000., diam
      write (9,1539) -dpdz/1000., diam
1539      format (/,10x,'specified pressure gradient',/, &
             10x,'pressure gradient = ',f7.2,' MPa/km',/, &
             10x,'initial conduit diameter = ',f10.2,' meters')
      end if

      if (ivt.eq.1) then
      write (6,1534)
      write (9,1534)
1534      format (/,10x,'no velocity adjustment',/)
      else
      write (6,1535)
      write (9,1535)
1535      format (/,10x,'automatic velocity adjustment',/)
      end if

      if (ives.eq.1) then
      write (6,1536)
      write (9,1536)
1536      format (/,10x, &
             'continued exsolution after fragmentation',/)
      else
      write (6,1537)
      write (9,1537)
1537      format (10x,'no exsolution after fragmentation',/)
      end if

      if (ifragtype.eq.1) then
      write (6,1542) fragval*100.
      write (9,1542) fragval*100.
1542      format (/,10x,'fragmentation at vesicularity of', &
             f6.1,'%'/)
      else if (ifragtype.eq.2) then
      write (6,1541)
      write (9,1541)
1541      format (/,10x, &
             'fragmentation when etz*dvdz>2.5 GPa',/)
      else if (ifragtype.eq.3) then
      write (6,1543)
      write (9,1543)
1543      format (/,10x, &
             'fragmentation when eta*dvdz>0.01*smod for ' &
             'vesicular solid')
      else
      write (6,1544)
      write (9,1544)
1544      format (/10x,'fragmentation when eta*dvdz*' &
             'stress concentration around bubble>250 MPa')
      end if


!      if (ivtype.eq.1) then
!      write (6,1545) 
!      write (9,1545)
!1545      format (/,10x,'vex = variable')
!      else
!      write (6,1543) vex
!      write (9,1543) vex
!1543      format (/,10x,'vex = ',f5.2)
!      end if

!      CALCULATE PARAMETERS AT BOTTOM OF CONDUIT

50      continue

      iruns = iruns+1

!      if (iruns.eq.20) stop

      write (6,1433)  iruns
      write (9,1433)  iruns
1433      format (' STARTING RUN NUMBER ',i3,':',$)

      p = pres(1)
      dz = -z(1)/50.
      wtmin = 5000.
      diam = dfirst
      temp = t1 + 273.15
      xsarea = pi*dfirst**2/4.

!      CALCULATE MASS FRACTION EXSOLVED GAS

      dh2o = .01*s*(p/1.0d+06)**beta
      if (dh2o.le.0.) dh2o = 1.0d-03
      mm = msil/(1.-dh2o)
      mg = 1.-mxtls-mm
      if (mg.lt.0.) then
      mg=0.
      mm = 1.-mxtls
      dh2o = mw/mm
      end if

!      CALCULATE MIXTURE DENSITY, VFGAS

      pbars = p/1.0e05
      call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
             entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
             dvmdt)
      if (mg.gt.0.) then
      call haarsi(temp,pbars,cpH2O,cvH2O,gH2O,hH2O, &
             sH2O,vH2O,blkgas,dvdth2o)
      rhof = (1.0D+06*.0180152)/vH2O
      rhomix = 1./(mm/rhom + mg/rhof + mxtls/rhoxtls)
      vfgas = mg*rhomix/rhof
      vxtls = mxtls*rhomix/rhoxtls
      vm = 1.-vfgas-vxtls 
      else
      rhomix = 1./(mm/rhom+mxtls/rhoxtls)
      vfgas = 0.
      vxtls = mxtls*rhomix/rhoxtls
      vm = 1.-vxtls
      end if

!      IF VFGAS > 0.75, ASSUME THAT THE MAGMA HAS ALREADY FRAGMENTED.

      if (vfgas.gt.fragval) then
      ifrag = 1
      zfrag = z(1)
      else
      ifrag = 0
      end if

!      STOP PROGRAM IF MAGMA PRESSURE GRADIENT > CONDUIT PRESSURE GRADIENT

      if ((icalc.eq.2).and.(rhomix.gt.(-dpdz/10.))) then
      write (6,1552) rhomix, rhomix/100., -dpdz/1000.
      write (9,1552) rhomix, rhomix/100., -dpdz/1000.
1552      format (/,10x,'WARNING!!',/,10x, &
             'Density of magma/gas mixture = ',f7.1,' kg/m3.',/, &
             10x,'Thus its pressure gradient is ',f7.1,' MPa/km.',/, &
             10x,'This is greater than that specified for the conduit:', &
             f7.1,/,10x, &
             'The conduit diameter will probably grow to an', &
             /,10x,' unrealistically large value before reaching',/, &
             10x,'the surface.  Do you wish to continue (y/n)?'$)
      read (5,1553) answer
1553      format(a1)
      if (answer.eq.'n') then
      write (6,*) 'program stopped'
      pause
      stop
      end if
      end if

!      CALCULATE INITIAL VALUES OF VARIABLES.      
!      IF ICALC=2, ADJUST INITIAL VELOCITY SO THAT 
!      DADZ IS INITIALLY = 0.

      mdot = rhomix*xsarea*vel(1)
      
      if (icalc.eq.1) then
      call derivs(z(1),pres(1),dpdz)
      dz = -0.2*p/dpdz
      else
      call derivs(z(1),xsarea,dadz)
      if ((dadz*dz).gt.(0.1*xsarea)) dz = xsarea*0.9/dadz
!      v = -diam**2/(32.*eta) * (dpdz + rhomix*g)
!      vel(1) = v
!      dadz = 0.
!      mdot = rhomix*xsarea*vel(1)
!      if (vel(1).lt.0.) then
!      write (6,*) 'dpdz is less than rhomix*g.  Program stopped'
!      pause
!      end if
      end if

      write (6,135) mdot
      write (9,135) mdot
135      format(10x,'      mass flux=',e12.4,' kg/s')

!      CALCULATE RESERVOIR ENTHALPY

      ho = mg*hf + mm*enthalpy + mxtls*(temp*cxtls+pres(1)/rhoxtls) +  &
             vel(1)**2/2.
      h = ho - vel(1)**2/2.      !current enthalpy
      area(1) = xsarea      !x-s area of conduit
      mach(1) = m      !mach number at base of conduit
      re(1) = rey      !reynolds number
      rho(1) = rhomix      !density
      time(1) = 0.      !time
      vesic(1) = vfgas      !volume fraction gas
      visc(1) = eta      !viscosity of mixture

!      ASSIGN VARIABLES TO OUTPUT ARRAY

      output(1,i) = area(i)
      output(2,i) = mach(i)
      output(3,i) = pres(i)/1.e+06
      output(4,i) = log10(re(i))
      output(5,i) = rho(i)
      output(6,i) = time(i)
      output(7,i) = vel(i)
      output(8,i) = vesic(i)
      output(9,i) = log10(visc(i))
      output(10,i) = z(i)
      output(11,i) = dadz
      output(12,i) = log10(pres(i))-6.0
      output(13,i) = dpdz*vel(i)/1.0e06_ip
      output(14,i) = dz
      output(15,i) = f
      output(16,i) = gamma
      output(17,i) = mg
      output(18,i) = mm
      output(19,i) = r
      output(20,i) = rhof
      output(21,i) = sv
      output(22,i) = temp-273.15
      output(23,i) = h/1.0d+03
      output(24,i) = cp
      output(25,i) = diam/2.
      output(26,i) = 100.*dh2o
      output(27,i) = cpm

!      WRITE VALUES TO OUTPUT FILES
      write (6,13) '   i', (title(ivar(ix)), ix=1,ivars)
      write (9,13) '   i', (title(ivar(ix)), ix=1,ivars)
      write (6,1402) i, (output(ivar(ix),1), ix=1,ivars)
      write (9,1402) i, (output(ivar(ix),1), ix=1,ivars)

      i = 1

!****************************************************************************

!      IF VESICULARITY=0, TEMPERATURE=CONSTANT, AND FLOW IS LAMINAR,
!      CALCULATE DEPTH AT WHICH GAS BEGINS TO
!      EXSOLVE USING THE ANALYTICAL EQUATION. 
!      USE THAT DEPTH AS THE STARTING POINT FOR NUMERICAL CALCULATIONS.

      if ((vfgas.eq.0.).and.(rey.lt.2000.).and.(itemp.eq.1)) then
      write (6,83)
      write (9,83)
83      format (/,5x,'base of conduit is below the depth gas ' &
             'begins to exsolve.',/, &
             5x,'Using analytical solution to calculate values below ' &
             'depth of saturation.',/)
      i = 2

!      CALCULATE PRESSURE AT WHICH GAS BEGINS TO EXSOLVE.

      p = 1.0d+06 * (100.*dh2o/s)**(1./beta)
      pres(2) = p

!      CALCULATE DEPTH AT WHICH GAS EXSOLVES
      z(2) = z(1) + (pres(2)-pres(1))/dpdz
      if (z(2).gt.0.) then
      z(2) = 0.
      pres(2) = pres(1) + (z(2)-z(1))/dpdz
      end if

!      ASSIGN NEW VALUES OF PARAMETERS TO ARRAYS

      if (icalc.eq.1) then
      call derivs(z(i),pres(i),dpdz)
      else
      call derivs(z(i),xsarea,dadz)
      end if

      area(i) = xsarea
      mach(i) = m
      re(i) = rey
      rho(i) = rhomix
      vel(i) = v
      vavg = (vel(i)+vel(i-1))/2.
      time(i) = time(i-1)+(z(i)-z(i-1))/vavg
      vesic(i) = vfgas
      visc(i) = eta

!      CALCULATE NEW ENTHALPY OF MIXTURE BY ASSUMING THAT THE SUM
!      (ENTHALPY + SP. KINETIC ENERGY OF MIXTURE) IS CONSTANT

      if (itemp.eq.2) then
      dh = - (vel(i)**2-vel(1)**2)/2. - g*(z(i)-z(1))
      h = ho + dh

!      CALL MULTISI, HAARSI, AND CALCULATE NEW ENTROPY, ENTHALPY
       pbars = p/1.0e05_8
112      call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
             entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
             dvmdt)

      if (p.gt.0.) then
      call haarsi(temp,pbars,cpH2O,cvH2O,gH2O,hH2O, &
             sH2O,vH2O,blkgas,dvdth2o)
      end if

      hf = hH2O/0.0180152
      cp = cpH2O/0.0180152

!      CALCULATE NEW ENTROPY, ENTHALPY OF MIXTURE
      hnew = mg*hf + mm*enthalpy  &
             + mxtls*(cxtls*temp + p/rhoxtls)
      cptotal = mg*cp + mm*cpm + mxtls*cxtls

!      IF ENTHALPY DOESN'T MATCH SPECIFIED VALUE, ADJUST T AND RECALCULATE
      if ((abs((h-hnew)/h)).gt.1.d-07) then
      temp = temp + (h-hnew)/cptotal
      go to 112
      end if
      end if

!      ASSIGN OUTPUT VALUES TO OUTPUT ARRAY

      output(1,i) = area(i)
      output(2,i) = mach(i)
      output(3,i) = pres(i)/1.e+06
      output(4,i) = log10(re(i))
      output(5,i) = rho(i)
      output(6,i) = time(i)
      output(7,i) = vel(i)
      output(8,i) = vesic(i)
      output(9,i) = log10(visc(i))
      output(10,i) = z(i)
      output(11,i) = dadz
      output(12,i) = log10(pres(i))-6.0
      output(13,i) = dpdz*vel(i)/1.0e06_ip
      output(14,i) = dz
      output(15,i) = f
      output(16,i) = gamma
      output(17,i) = mg
      output(18,i) = mm
      output(19,i) = r
      output(20,i) = rhof
      output(21,i) = sv
      output(22,i) = temp-273.15
      output(23,i) = h/1.0d+03
      output(24,i) = cp
      output(25,i) = diam/2.
      output(26,i) = 100.*dh2o
      output(27,i) = cpm

      write (6,1402) i, (output(ivar(ix),i), ix=1,ivars)
      write (9,1402) i, (output(ivar(ix),i), ix=1,ivars)
      end if

!**********************************************************************

!      BEGIN LOOP

100      continue

      zin = z(i)
      p = pres(i)
      if ((icalc.eq.1).and.((p+dz*dpdz).lt.0.)) dz = -0.2*p/dpdz

!      CALL SELF-ADJUSTING RUNGE-KUTTA PROGRAM TO CALCULATE NEXT Z STEP
!      AND PRESSURE ETC AT THAT STEP.

      if (icalc.eq.1) then
      call rkqs(p,dpdz,1,zin,dz,eps,p,dzout,dznext,derivs)
      if (p.lt.0.) then
      dz= -0.2*pres(i)/dpdz
      go to 100
      end if
      i = i+1
      z(i) = zin 
      pres(i) = p
      dz = dznext
      else
      call bsstep(xsarea,dadz,1,zin,dz,eps,xsarea,dzout,dznext,derivs)

!      STOP PROGRAM IF CONDUIT RADIUS GETS UNREALISTICALLY LARGE.
      if ((xsarea.gt.8.d+05).or.(dadz.gt.1.d+50)) then
      write (6,5162)
5162      format(/,5x,'Conduit diameter or dadz ', &
             'is unrealistically large.',/, &
             5x,'Program stopped.',/)
      go to 1501
      end if

      if (xsarea.lt.0.) then
      dz = dz/2.
      if (dz.lt.epsilon(dz)) pause 'minimum step size reached'
      xsarea = area(i)
      go to 100
      end if
      i = i+1
      z(i) = zin
      pres(i) = pfinal + dpdz*(zin-zfinal)
      dz = dznext
      if ((xsarea+dz*dadz).lt.0.) dz = -0.9*xsarea/dadz
      end if

      if ((i.eq.1).and.(dzout.ne.dz)) output(14,1) = dzout

!      CHECK TO SEE IF WE'VE REACHED THE FRAGMENTATION DEPTH YET.
!      THE VARIABLES ARE AS FOLLOWS:
!      DVDZ = EXTENSIONAL STRAIN RATE IN THE CONDUIT.  THE EQUATION 
!      BELOW USES THE BULK VALUE.  HOWEVER YOU MAY WISH TO USE
!      THE MAXIMUM EXTENSIONAL STRAIN RATE, WHICH IS CALCULATED
!      AS EMAX IN THE SUBROUTINE "DERIVS".  EMAX IS THE EXTENSIONAL
!      STRAIN RATE AT THE CONDUIT WALL, WHICH IS SIGNIFICANTLY
!      HIGHER THAN THE AVERAGE EXTENSIONAL STRAIN RATE.
!      SMOD = ELASTIC MODULUS.  IT IS CURRENTLY SET AT A CONSTANT VALUE.  
!      HOWEVER IT SHOULD DECREASE WITH INCREASING GAS-VOLUME FRACTION.
!      ONE EXAMPLE OF A RELATION FOR ELASTIC MODULUS THAT DECREASES
!      WITH VOLUME FRACTION COMES FROM GIBSON, L.J, AND ASHBY, M.F.,
!      'CELLULAR SOLIDS, PERGAMON PRESS, P. 79.  USING THEIR RELATION,
!      AND ASSUMING THE VESICULAR MELT BEHAVES (OVER SHORT TIME SCALES)
!      LIKE A 2-DIMENSIONAL HONEYCOMB SOLID, THE ELASTIC MODULUS (E) WOULD
!      BE E/ES = 2.3*(T/L)^3, WHERE ES IS THE YOUNG'S MODULUS OF THE 
!      SOLID MATRIX, T IS THE THICKNESS OF THE HONEYCOMB WALLS AND L IS THE
!      LENGTH OF EACH STRAIGHT SEGMENT IN THE HONEYCOMB WALLS.  THE RELATION
!      BETWEEN T/L AND THE VOLUME FRACTION OF VOIDS IS VFGAS=(1-(T/(L*2SIN(PI/3))))^2.
!      ETA = VISCOSITY OF MIXTURE.  IN THIS PROGRAM, THE VISCOSITY DEPENDS ON GAS VOLUME FRACTION
!      ACCORDING TO THE CAPILLARY NUMBER OF THE BUBBLES.  PAPALE (1999) USES
!      ETA = ETAMAG/(1-VFGAS), WHERE ETAMAG IS THE VISCOSITY OF THE MELT+CRYTALS.
!
      if (ifrag.eq.0) then
      select case (ifragtype)
      case (1)
      if (vfgas.gt.fragval) ifrag=1
      zfrag = z(i)
      case (2)
!      mixture viscosity * strain rate > 25 GPa
      if ((eta*dvdz).gt.2.5d+07) ifrag=1
      zfrag = z(i)
      case (3)
!      The formula below is the elastic modulus of the vesicular mixture,
!      corrected for vesicularity using a formula derived from 2-D boundary-element
!      modeling of the compressibility of a hexagonal unit cell of melt containing
!      a central bubble of varying diameter, under uniaxial loading.
      smod = 2.5d+10*(1-2.1*vfgas + 0.81*vfgas**2 + &
             0.98*vfgas**3 - 0.69*vfgas**4)      !elastic modulus
      if (dvdz.gt.(0.01*smod/eta)) then 
      ifrag=1
      end if
      zfrag = z(i)
      case (4)
!      The following formula is for the maximum stress around a circular hole,
!      normalized to a uniaxial far-field stress, as a function of volume fraction
!      of holes in a cellular solid.  It was derived by boundary-element modeling
      stresscon = 1100.*vfgas**5-1730.*vfgas**4+1060.*vfgas**3- &
             261.*vfgas**2+29.3*vfgas+2.64
      if ((eta*dvdz*stresscon).gt.2.5d+07) ifrag=1
      zfrag = z(i)
      end select
      end if

!      IF Z>ZFINAL, ADJUST DZ AND CALL RK4 TO CALCULATE P AT Z=ZFINAL

      if (z(i).gt.zfinal) then
      dz=zfinal-z(i-1)
      if (icalc.eq.1) then
      call rkck(pres(i-1),dpdz,1,z(i-1),dz,pout,perr,derivs)
      pres(i) = pout
      else
      call rkck(area(i-1),dadz,1, z(i-1),dz,xsarea,xserr,derivs)
      pres(i) = pfinal
      area(i) = xsarea
      end if
      z(i) = zfinal
      end if

!      ASSIGN NEW VALUES OF PARAMETERS TO ARRAYS

      if (icalc.eq.1) then
      call derivs(z(i),pres(i),dpdz)
      else
      call derivs(z(i),xsarea,dadz)
      end if

      area(i) = xsarea
      mach(i) = m
      re(i) = rey
      rho(i) = rhomix
      vel(i) = v
      vavg = (vel(i)+vel(i-1))/2.
      time(i) = time(i-1)+(z(i)-z(i-1))/vavg
      vesic(i) = vfgas
      visc(i) = eta

!      CALCULATE NEW ENTHALPY OF MIXTURE BY ASSUMING THAT THE SUM
!      (ENTHALPY + SP. KINETIC ENERGY OF MIXTURE) IS CONSTANT

      if (itemp.eq.2) then
      dh = - (vel(i)**2-vel(1)**2)/2. - g*(z(i)-z(1))
      h = ho + dh

!      CALL MULTISI, HAARSI, AND CALCULATE NEW ENTROPY, ENTHALPY
         pbars = p/1.0e05_8
111      call multisi(temp,pbars,blkmag,cpm,dh2o,enthalpy, &
             entropy,mwttot,rhom,wtoxan,xcomp,hwm,swm,gwm,dgdxw, &
             dvmdt)
      if (p.gt.0.) then
      pbars = p/1.0e05_8
      call haarsi(temp,pbars,cpH2O,cvH2O,gH2O,hH2O,sH2O, &
             vH2O,blkgas,dvdth2o)
      end if

      hf = hH2O/0.0180152
      cp = cpH2O/0.0180152

!      CALCULATE NEW ENTROPY, ENTHALPY OF MIXTURE
      hnew = mg*hf + mm*enthalpy + mxtls*(cxtls*temp + p/rhoxtls)
      cptotal = mg*cp + mm*cpm + mxtls*cxtls

!      IF ENTHALPY DOESN'T MATCH SPECIFIED VALUE, ADJUST T AND RECALCULATE
      if ((abs((h-hnew)/h)).gt.1.d-07) then
      temp = temp + (h-hnew)/cptotal
      go to 111
      end if

      end if

!      WRITE VALUES TO ARRAY "OUTPUT"

      output(1,i) = area(i)
      output(2,i) = mach(i)
      output(3,i) = pres(i)/1.e+06
      output(4,i) = log10(re(i))
      output(5,i) = rho(i)
      output(6,i) = time(i)
      output(7,i) = vel(i)
      output(8,i) = vesic(i)
      output(9,i) = log10(visc(i))
      output(10,i) = z(i)
      output(11,i) = dadz
      output(12,i) = log10(pres(i))-6.0
      output(13,i) = dpdz*vel(i)/1.0e06_ip
      output(14,i) = dz
      output(15,i) = f
      output(16,i) = gamma
      output(17,i) = mg
      output(18,i) = mm
      output(19,i) = r
      output(20,i) = rhof
      output(21,i) = sv
      output(22,i) = temp-273.15
      output(23,i) = ho/1.0d+03
      output(24,i) = cp
      output(25,i) = diam/2.
      output(26,i) = 100.*dh2o
      output(27,i) = cpm

!      WRITE OUT EVERY 20TH DATA POINT TO THE SCREEN
      if ((i/20).eq.(float(i)/20.)) then
      write (6,1402) i, (output(ivar(ix),i), ix=1,ivars)
      end if

!      CALCULATE AVERAGE SHEAR STRAIN RATE IN CONDUIT
      edot = 4.*v/(3.*diam)

!      CALCULATE WATER TABLE DEPTH REQUIRED FOR HYDROSTATIC PRESSURE
!      TO EXCEED PRES(I) (ASSUME WATER DENSITY OF 0.9, OR 9000 PA/M)

      wtdepth = z(i) + (pres(i)-1.013d+05)/10000.
      if (wtdepth.lt.wtmin) wtmin=wtdepth

      if (icalc.eq.2) go to 1500

!      IF ICALC=1 AND P<PFINAL, OR M>1, ADJUST THE INPUT VELOCITY

      if (((pres(i).lt.pfinal).or.(mach(i).gt.1.00)) &
             .and.(z(i).lt.(zfinal-0.5))) then
      if (ivt.eq.1) then
1491      if (pres(i).lt.pfinal) then
      write (6,533)
      write (9,533)
533      format (/,5x,'p < pfinal.  program stopped')
      else
      write (6,1434)
      write (9,1434)
1434      format (/,5x,'mach number > 1.  program stopped')
      end if
      go to 1501
      else
1492      write (6,1402) i, (output(ivar(ix),i), ix=1,ivars)
      write (9,1402) i, (output(ivar(ix),i), ix=1,ivars)
      if ((vel(1).eq.0.001).and.(pres(i).lt.pfinal)) then
      write (6,1445)
      write (9,1445)
1445      format(/,10x,'pressure insufficient to produce eruption',/)
      iend = 1
      go to 1501
      end if
      call adjust
      if (iend.eq.1) go to 1501
      go to 50
      end if
      end if

!      IF M>0.99999 AND Z>-(zfinal-2.), CALL IT GOOD

      if ((z(i).gt.(zfinal-2.)).and.(mach(i).gt.0.9999)) go to 1501

!      RETURN TO BEGINNING OF LOOP IF Z < ZFINAL. AND P > PFINAL

1500      if ((z(i).lt.zfinal).and.(i.lt.imax)) go to 100

!      IF imax ITERATIONS IS REACHED, OR IVT=1, STOP PROGRAM

      if (i.eq.imax) then 
      write (6,5982)
5982      format(5x,'maximum number of iterations exceeded.', &
             ' program stopped')
      go to 1501
      end if
      if (ivt.eq.1) go to 1501

!      IF Z=zfinal AND P>PFINAL, ADJUST THE INPUT VELOCITY AND START OVER

      if (pres(i).gt.(pfinal+1000.)) then
      write (6,1402) i, (output(ivar(ix),i), ix=1,ivars)
      write (9,1402) i, (output(ivar(ix),i), ix=1,ivars)
      call adjust
      if (iend.eq.1) then
      write (6,5109)
      write (9,5109)
5109      format (/,10x,'limit of resolution reached',/)
      go to 1501
      end if
      go to 50

!      IF PRESSURE IS WITHIN 0.0001 MPA OF PFINAL, CALL IT GOOD

      else if (abs(pfinal-pres(i)).lt.1000.) then
      go to 1501

!      IF PRESSURE IS OUTSIDE THIS TOLERANCE LEVEL,
!      ADJUST THE INPUT VELOCITY

      else
      write (6,1402) i, (output(ivar(ix),i), ix=1,ivars)
      write (9,1402) i, (output(ivar(ix),i), ix=1,ivars)
      write (6,1424)
      write (9,1424)
1424      format (/,10x,'pressure < pfinal',/)
      call adjust
      if (iend.eq.1) go to 1501
      go to 50
      end if

!***********************************************************************

!      ONCE THE A SOLUTION HAS BEEN REACHED, WRITE OUT FINAL RESULTS

1501      continue

      ifinal = i

!      WRITE OUT HEADERS
13      format (/,1x,a5,7a10)

!      WRITE FINAL VALUES TO STANDARD OUTPUT
      write (6,1402) ifinal, (output(ivar(ix),ifinal), ix=1,ivars)
      write (9,1402) ifinal, (output(ivar(ix),ifinal), ix=1,ivars)
      if ((ifinal.eq.imax).and.(icalc.eq.2)) then
      write (6,1902)
      write (9,1902)
1902      format (/,10x,'Number of iterations has exceeded the limit.',/, &
             10x,'You should be able to perform this calculation in',/, &
             10x,'fewer iterations.  If icalc=2,Increasing the ',/, &
             10x,'initial conduit radius or decreasing the initial',/, &
             10x,'velocity may reduce the required # of iterations.')
      go to 1034
      end if
      write (6,1401)
      write (9,1401)
1401      format (/,' successful completion')


!*************************************************************************

!      CALCULATE MAXIMUM THEORETICAL VELOCITY, FINAL TEMPERATURES
      gmix = (mg*cp + mm*cpm + mxtls*cxtls)/ &
             (mg*cv + mm*cpm + mxtls*cxtls)

!      2.  CALCULATE NEW TEMPERATURE:
      temp2 = temp*(pfinal/pres(i))**((gmix-1.)/gmix)
      temp2c = temp2-273.15
      deltat = temp-temp2

!      3.  CALCULATE CHANGE IN ENTHALPY

      deltah = mg*cp*deltat  &
             + mm*(cpm*deltat + (pres(i)-pfinal)/rhom) &
             + mxtls * (cxtls*deltat+(pres(i)-pfinal)/rhoxtls)

!      4.  CALCULATE MAXIMUM THEORETICAL VELOCITY

      if (deltah.ge.0.) then
      vmax = vel(i) + sqrt(2.*deltah)
      else
      vmax = vel(i) - sqrt(-2.*deltah)
      end if

      write (6,1432)  pmin, temp2c, deltat, deltah, vmax
      write (9,1432)  pmin, temp2c, deltat, deltah, vmax
1432      format(/,5x,'AFTER ISENTROPIC EQUILIBRATION TO',f10.4, &
             ' MPa PRESSURE:',/, &
             10x,' final temperature = ',f7.2,' deg. C',/, &
             10x,' temperature change = ',f9.3,' deg. K',/, &
             10x,' enthalpy change = ',e12.4,' J/kg',/, &
             10x,' max. theoretical velocity = ',f9.2,' m/s',/)

!      WRITE OUT WATER TABLE DEPTH REQUIRED FOR WATER INFLUX

      write (6,1042) wtmin
      write (9,1042) wtmin
1042      format (/,' maximum water table depth that will', &
             ' allow g.w. influx = ',f8.2,' meters',/, &
             5x,'Negative values are below the ground surface,',/, &
             5x,'positive values are above.',/, &
             5x,'See caveats on p. 43-44 of model documentation.',/)

!      CALCULATE LENGTH OF UNDERPRESSURED INTERVAL

!      if (icalctxt.eq.'diam') then 
!      call hydro2
!      write (6,1493) mdot
!      write (9,1493) mdot
!1493      format (5x,'This compares with a magma-flux rate of ',
!             e12.4,' kg/s',/)
!      end if

!      WRITE RESULTS TO OUTPUT FILE

1034      write (9,135) mdot

      write (9,157)
157      format (/,'OUTPUT TABLE')

      iwrite = ifinal/200
      if (iwrite.lt.1) iwrite=1
1301      format (i4,a3,7a10)

      if (icalc.eq.1) then      !if icalc=1

!      WRITE OUT TITLE LINE
      itotal = ifinal/iwrite      !ensure that last point is included
      if ((ifinal/iwrite).lt.(float(ifinal)/float(iwrite))) then
      itotal = itotal + 1
      endif
      write (9,1301) itotal,'  i', (title(ivar(ix)), ix=1,ivars)

!      WRITE OUT DATA
      do 1100 in=1,ifinal,iwrite
1100      write (9,1403) in, (output(ivar(ix),in), ix=1,ivars)

      if (in.lt.(ifinal+iwrite)) then
      write (9,1403) ifinal, (output(ivar(ix),ifinal), ix=1,ivars)
      end if

      else if (icalc.eq.2) then      !if icalc=2

!      WRITE OUT TITLE LINE
      itotal = 0      !calculate the number of points written out
      do 1111 in=1,ifinal,iwrite      
      if (z(in).ge.zfrag) go to 1112
      itotal = itotal+1
1111      continue
1112      continue
      do 1113 im=in-iwrite+1,ifinal      
      itotal = itotal+1
1113      continue
      write (9,1301) itotal,'  i', (title(ivar(ix)), ix=1,ivars)

!      WRITE OUT DATA
      do 1101 in=1,ifinal,iwrite      !below fragmentation depth, skip some points
      if (z(in).ge.zfrag) go to 1102
      write (9,1403) in, (output(ivar(ix),in), ix=1,ivars)
1101      continue
1102      continue
      do 1103 im=in-iwrite+1,ifinal      !above fragmentation depth, write out every point
      write (9,1403) im, (output(ivar(ix),im), ix=1,ivars)
1103      continue
      end if

1402  format (1x,i5,7f10.3)
1403  format (3x,i4,7f10.3)

      close(8)
      close(9)
      close(10)

      end
