       subroutine vis

!       SUBROUTINE TO CALCULATE VISCOSITY OF THE MELT+CRYSTALS
!       AS A FUNCTION OF MELT COMPOSITION AND TEMPERATURE

!       DECLARE VARIABLES USED IN THIS SUBROUTINE

      use precis_param
      use Common1
      use Common2
      use Common3
      use ints
      use melt
      use water

      implicit none

      real(kind=ip) :: ceta,corrfact,ct, &
                etamaglg,slope,tmoles,totalx
      real(kind=ip) ::  mAl2O3,mCaO,mFe2O3,mFeO, &
                mK2O,mMgO,mNa2O,mSiO2,mTiO2, &
                sAl2O3,sCaO,sFe2O3,sFeO,sK2O,sMgO, &
                sNa2O,sSiO2,sTiO2, &
                xAl2O3,xCaO,xFe2O3,xFeO,xH2O,xK2O, &
                xMgO,xNa2O,xSal,xSiO2,xTiO2

!       DECLARE VARIABLES LISTED IN THE COMMON BLOCK BUT NOT USED
!       IN THIS SUBROUTINE

       real(kind=ip) ::  gamma1

        if (wtoxan(1).gt.70.) then

!         IF THE WT% ANHYDROUS SIO2 IN THE MELT IS GREATER THAN 70%, 
!         CALCULATE VISOCISTY USING RELATIONS FROM 
!         HESS AND DINGWELL (1996):
                
                etamaglg = (-3.545+0.833*log(dh2o*100.)) &
                          + (9601.-2368.*log(dh2o*100.)) &
                         / (temp - (195.7 + 32.25*log(dh2o*100.)))
                etamag = 10.**etamaglg
          else

!         IF THE WT% SIO2 < 70%, USE RELATIONS FROM SHAW (1972).

!         THE VARIABLES MSIO2, MAL2O3, ETC. ARE THE 
!         MOLECULAR WEIGHTS OF EACH OXIDE (KG/MOLE), WITH THE 
!         EXCEPTIONS THAT
!         MAL2O3 AND MFE2O3 ARE PER GRAM-ATOM AL (I.E. HALF 
!         THEIR MOLECULAR WEIGHTS), FOLLOWING THE METHODOLOGY 
!         OF SHAW (1972).

                ceta = -6.40
                ct = 1.50
                mSiO2= .060084
                mAl2O3 = .050981
                mFe2O3 = .079845
                mFeO = .071846
                mMgO = .040304
                mCaO = .056077
                mTiO2 = .079878
                mNa2O = .061979
                mK2O = .094195
                mH2O = .018015

!       DECLARE VALUES OF EMPIRICAL COEFFICIENTS "Si" FROM SHAW (1972)
!       THESE CONSTANTS ARE USED TO DETERMINE THE SLOPE OF THE LN(VISCOSITY)
!       VERSUS TEMPERATURE CURVE FOR THE MAGMA.

                sAl2O3= 6.7
                sFe2O3 = 3.4
                sFeO = 3.4
                sMgO = 3.4
                sCaO = 4.5
                sTiO2 = 4.5
                sNa2O = 2.8
                sK2O = 2.8
                sH2O = 2.

!       CALCULATE TOTAL NUMBER OF MOLES OF OXIDES

                corrfact = 1.-dh2o
                tmoles = (SiO2/mSiO2 + Al2O3/mAl2O3 + Fe2O3/mFe2O3  &
                + FeO/mFeO + MgO/mMgO + CaO/mCaO + TiO2/mTiO2 &
                + Na2O/mNa2O + K2O/mK2O) * corrfact &
                + 100.*dh2o/mH2O

!       CALCULATE MOLE FRACTIONS OF EACH OXIDE

                xSiO2 = corrfact*SiO2/(mSiO2*tmoles)
                xAl2O3 = corrfact*Al2O3/(mAl2O3*tmoles)
                xFe2O3 = corrfact*Fe2O3/(mFe2O3 *tmoles)
                xFeO = corrfact*FeO/(mFeO*tmoles)
                xMgO = corrfact*MgO/(mMgO*tmoles)
                xCaO = corrfact*CaO/(mCaO*tmoles)
                xTiO2 = corrfact*TiO2/(mTiO2*tmoles)
                xNa2O = corrfact*Na2O/(mNa2O*tmoles)
                xK2O = corrfact*K2O/(mK2O*tmoles)
                xH2O =  100.*dh2o/(mH2O*tmoles)

!               MAKE SURE MOLE FRACTIONS ALL ADD UP TO 1.

                totalx = xSiO2+xAl2O3+xFe2O3+xFeO+xMgO+xCaO+xTiO2+xNa2O+ &
                xK2O+xH2O

!               CALCULATE SLOPE OF ARRHENIAN CURVE FOR THE SHAW RELATION

                xsal = xSiO2/(1.-xSiO2)
                slope = sAl2O3*xAl2O3*xsal + sFe2O3*xFe2O3*xsal +  &
                   sFeO*xFeO*xsal + sMgO*xMgO*xsal + sCaO*xCaO*xsal +  &
                   sTiO2*xTiO2*xsal + sNa2O*xNa2O*xsal + sK2O*xK2O*xsal + &
                   sH2O*xH2O*xsal

!               CALCULATE VISCOSITY 
!               SHAW'S CONSTANTS ARE VALID IF THE VISCOSITY IS IN POISE,
!               AND THE EQUATION IS FOR THE NATURAL LOG (NOT LOG(10)).

                etamaglg = slope*1.0e+04/temp - ct*slope + ceta
                etamag = exp(etamaglg)

!               CONVERT VISCOSITY FROM POISE TO PA S.

                etamag = etamag/10.
                etamaglg = log10(etamag)
        end if

!        2.  CALCULATE VISCOSITY OF LIQUID PLUS CRYSTALS 
!       USING EINSTEIN-ROSCOE
!       EQUATION:       ETA = ETALIQ*(1.-PHI/0.6)**(-2.5)

        if (xtls.lt.59.) then
          etamag = etamag*(1.-xtls/60.)**(-2.5)
        end if

        return          
        end
